<?php

namespace Drupal\earkiv;

use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\Query\QueryInterface;
use Drupal\Core\Link;
use Drupal\Core\StringTranslation\ByteSizeMarkup;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a list controller for the 'earkiv_archive' entity.
 */
class ArchiveListBuilder extends EntityListBuilder {

  protected DateFormatterInterface $dateFormatter;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    EntityTypeInterface $entity_type,
    EntityStorageInterface $storage,
    DateFormatterInterface $date_formatter
  ) {
    parent::__construct($entity_type, $storage);
    $this->dateFormatter = $date_formatter;
  }

  /**
   * {@inheritdoc}
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    return new static(
      $entity_type,
      $container->get('entity_type.manager')->getStorage($entity_type->id()),
      $container->get('date.formatter')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function render() {
    $build = parent::render();
    $build['table']['#empty'] = $this->t('There are no E-Arkiv archives yet.');
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header['id'] = $this->t('ID');
    $header['file'] = $this->t('File');
    $header['size'] = $this->t('Size');
    $header['status'] = $this->t('Status');
    $header['created'] = $this->t('Created');
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    /* @var $entity \Drupal\earkiv\Entity\Archive */
    $row['id'] = $entity->id();
    $row['file']['data'] = $this->getFilename($entity);
    $row['size'] = $this->getFileSize($entity);
    $row['status'] = ArchiveStatusEnum::getStatusText($entity->getStatus());
    $row['created'] = $this->dateFormatter->format($entity->getCreatedTime(), 'short');
    return $row + parent::buildRow($entity);
  }

  /**
   * Get the archive name with a link to download it.
   */
  protected function getFilename(EntityInterface $entity): array {
    /** @var \Drupal\file\FileInterface $file */
    $file = $entity->getFile();
    if (is_null($file)) return ['#markup' => $this->t('Not available')];
    return [
      '#type' => 'link',
      '#title' => $file->getFilename(),
      '#url' => \Drupal::service('file_url_generator')->generate($file->getFileUri()),
    ];
  }

  /**
   * Get formatted file size.
   */
  protected function getFileSize(EntityInterface $entity): string {
    /** @var \Drupal\file\FileInterface $file */
    $file = $entity->getFile();
    $size = '-';
    if (!is_null($file)) $size = ByteSizeMarkup::create($file->getSize());;
    return $size;
  }

  /**
   * Returns a query object for loading entity IDs from the storage.
   *
   * @return \Drupal\Core\Entity\Query\QueryInterface
   *   A query object used to load entity IDs.
   */
  protected function getEntityListQuery(): QueryInterface {
    $query = $this->getStorage()->getQuery()
      ->accessCheck(TRUE)
      ->sort('created', 'DESC');

    // Only add the pager if a limit is specified.
    if ($this->limit) {
      $query->pager($this->limit);
    }
    return $query;
  }

}
