<?php

namespace Drupal\earkiv\Drush\Commands;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\earkiv\ArchiverPluginManager;
use Drupal\earkiv\ArchiveStatusEnum;
use Drupal\earkiv\Entity\Archive;
use Drupal\earkiv\JobStatusTrait;
use Drupal\earkiv\StoragePluginManager;
use Drush\Attributes as CLI;
use Drush\Commands\DrushCommands;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class EArkivCommands.
 */
final class EArkivCommands extends DrushCommands {

  use JobStatusTrait;

  protected EntityTypeManagerInterface $entityTypeManager;
  protected FileSystemInterface $fileSystem;
  protected ArchiverPluginManager $archiverPluginManager;
  protected StoragePluginManager $storagePluginManager;
  protected LoggerChannelInterface $log;

  /**
   * HTTrackCommands constructor.
   */
  final public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    FileSystemInterface $file_system,
    ArchiverPluginManager $archiver_plugin_manager,
    StoragePluginManager $storage_plugin_manager,
    LoggerChannelFactoryInterface $logger_channel_factory
  ) {
    parent::__construct();
    $this->entityTypeManager = $entity_type_manager;
    $this->fileSystem = $file_system;
    $this->archiverPluginManager = $archiver_plugin_manager;
    $this->storagePluginManager = $storage_plugin_manager;
    $this->log = $logger_channel_factory->get('earkiv');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('file_system'),
      $container->get('plugin.manager.earkiv_archiver'),
      $container->get('plugin.manager.earkiv_storage'),
      $container->get('logger.factory')
    );
  }

  /**
   * Run HTTrack processing.
   */
  #[CLI\Command(name: 'earkiv:process')]
  #[CLI\Usage(name: 'earkiv:process', description: 'Process E-Arkiv jobs.')]
  public function process(): void {
    // Make sure no archiving process is running.
    if ($this->jobInProgress()) return;

    // Get next job.
    $entity = $this->getNextJob();

    // Make sure we got a job to process.
    if (!$entity) {
      $this->output()->writeln('No jobs to process.');
      return;
    }

    // What step are we on?
    $status = $entity->getStatus();
    try {
      switch ($status) {
        case ArchiveStatusEnum::NEW:
        case ArchiveStatusEnum::RESUME_ARCHIVE:
          $plugin = $this->archiverPluginManager->createInstance($entity->getArchiverPlugin());;
          $command = $plugin->getDrushCommands();
          $command->process($entity, $this->output());
          break;
        case ArchiveStatusEnum::WAITING:
        case ArchiveStatusEnum::RESUME_STORE:
          $plugin = $this->storagePluginManager->createInstance($entity->getStoragePlugin());
          $command = $plugin->getDrushCommands();
          $command->process($entity, $this->output());
          break;
        default:
          $this->output()->writeln(sprintf('Unexpected status: %d.', $status));
      }
    }
    catch (\Exception $e) {
      $this->log->error('Error processing job: %message', ['%message' => $e->getMessage()]);
      $this->setJobStatusErrorResume($entity);
    }
  }

  /**
   * Is there a processing job in progress?
   */
  protected function jobInProgress(): bool {
    $query = $this->entityTypeManager->getStorage('earkiv_archive')->getQuery();
    $or = $query->orConditionGroup();
    $or->condition('status', ArchiveStatusEnum::RUNNING_ARCHIVE);
    $or->condition('status', ArchiveStatusEnum::RUNNING_STORE);
    $count = $query->accessCheck(FALSE)
      ->condition($or)
      ->count()
      ->execute();
    return $count > 0;
  }

  /**
   * Get the next job for processing.
   */
  protected function getNextJob(): Archive|NULL {
    $query = $this->entityTypeManager->getStorage('earkiv_archive')->getQuery();
    $or = $query->orConditionGroup();
    $or->condition('status', ArchiveStatusEnum::NEW);
    $or->condition('status', ArchiveStatusEnum::RESUME_ARCHIVE);
    $or->condition('status', ArchiveStatusEnum::WAITING);
    $or->condition('status', ArchiveStatusEnum::RESUME_STORE);
    $ids = $query->accessCheck(FALSE)
      ->condition($or)
      ->sort('created', 'ASC')
      ->range(0, 1)
      ->execute();
    if (empty($ids)) return NULL;

    // Load entity.
    $id = reset($ids);
    return Archive::load($id);
  }

}
