<?php

namespace Drupal\earkiv\Drush\Commands;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\earkiv\ArchiverPluginManager;
use Drupal\earkiv\ArchiveStatusEnum;
use Drupal\earkiv\EArkivDrushCommandsInterface;
use Drupal\earkiv\Entity\Archive;
use Drupal\earkiv\JobStatusTrait;
use Drupal\file\Entity\File;
use Drush\Commands\DrushCommands;
use Symfony\Component\Console\Helper\ProgressIndicator;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Process\Process;

/**
 * Class HTTrackCommands.
 */
final class HTTrackCommands extends DrushCommands implements EArkivDrushCommandsInterface {

  use JobStatusTrait;

  protected EntityTypeManagerInterface $entityTypeManager;
  protected FileSystemInterface $fileSystem;
  protected ArchiverPluginManager $pluginManager;
  protected LoggerChannelInterface $log;

  /**
   * HTTrackCommands constructor.
   */
  final public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    FileSystemInterface $file_system,
    ArchiverPluginManager $plugin_manager,
    LoggerChannelFactoryInterface $logger_channel_factory
  ) {
    parent::__construct();
    $this->entityTypeManager = $entity_type_manager;
    $this->fileSystem = $file_system;
    $this->pluginManager = $plugin_manager;
    $this->log = $logger_channel_factory->get('earkiv');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('file_system'),
      $container->get('plugin.manager.earkiv_archiver'),
      $container->get('logger.factory')
    );
  }

  /**
   * Process job.
   */
  public function process(Archive $entity, OutputInterface $output): void {
    $this->setOutput($output);

    // Get the archiver plugin.
    $archiver = $this->pluginManager->createInstance($entity->getArchiverPlugin());
    $archiver->setConfiguration($entity->getArchiverConfig());;

    // Create the working directory.
    $tmp_dir = $this->fileSystem->getTempDirectory();
    $work_dir = 'httrack_' . $entity->id();
    $dir = $tmp_dir . '/' . $work_dir;
    $dir_created = $this->fileSystem->prepareDirectory($dir, FileSystemInterface::CREATE_DIRECTORY);

    // If the directory was not created, exit with an error.
    if (!$dir_created) {
      throw new \Exception('Could not create working directory.');
    }

    // Create an empty tar archive.
    $filename = sprintf('earkiv_%d_%d.tar', $entity->id(), $entity->getCreatedTime());
    $command = ['tar', '-cf', $filename, '-T', '/dev/null'];
    $process = new Process($command, $dir);
    $process->mustRun();

    // Create a managed file.
    $file = $entity->getFile();
    if (is_null($file)) {
      $file = File::create([
        'filename' => $filename,
        'uri' => 'temporary://' . $work_dir . '/' . $filename,
        'status' => 1,
        'uid' => $entity->get('uid')->target_id,
      ]);
      $file->save();

      // Update the archive entity with file ID.
      $entity->set('file', $file->id());
    }

    // Update Archive entity.
    $entity->set('status', ArchiveStatusEnum::RUNNING_ARCHIVE);
    $entity->save();

    // Define the httrack command.
    $command = [
      'httrack',
      $archiver->getConfiguration()['url'],
      '--path',
      $dir,
      '-N',
      '%h%p/%n/index%[page].%t',
      '--userdef-cmd',
      sprintf('tar --transform "s:%s/::g" --append --file=%s \$0', ltrim($dir, '/'), $filename),
      '--mirror',
      '--quiet',
      '--do-not-log',
      '--robots=0',
      '--footer',
      '',
    ];

    // Create a process.
    $process = new Process($command, $dir, NULL, NULL, NULL);

    // Create progress indicator.
    $this->output()->setVerbosity(OutputInterface::VERBOSITY_VERY_VERBOSE);
    $progress = new ProgressIndicator($this->output());
    $progress->start('Crawling...');

    // Run the process.
    $process->start();

    // Advance progress indicator while the process is running.
    while ($process->isRunning()) {
      // TODO: Read hts-in_progress.lock to check on progress for the command. Is this useful?
      // TODO: Create a hts-stop.lock to pause the command. Remove it to continue? Should we use this?
      $progress->advance();
    }

    // Handle process result.
    if ($process->isSuccessful()) {
      // Compress the tar archive.
      $command = ['gzip', $filename];
      $gzip_process = new Process($command, $dir);
      $gzip_process->start();
      $progress->setMessage('Compressing...');
      while ($gzip_process->isRunning()) {
        $progress->advance();
      }

      if ($gzip_process->isSuccessful()) {
        // Update URI and filename.
        $file->setFileUri($file->getFileUri() . '.gz');
        $file->setFilename($file->getFilename() . '.gz');
        $file->save();

        // Signal that the archive has been processed and is ready for storage.
        $entity->set('status', ArchiveStatusEnum::WAITING);
        $entity->save();

        // Let the user know we are done.
        $progress->finish('Finished.');
        $this->output()->writeln('Archive created successfully.');
      }
      else {
        $this->log->error('Failed to compress archive: @message', ['@message' => $gzip_process->getErrorOutput()]);
        $this->setJobStatusErrorResume($entity);
      }
    }
    else {
      $this->log->error('Failed to create archive: @message', ['@message' => $process->getErrorOutput()]);
      $this->setJobStatusErrorResume($entity);
    }
  }

}
