<?php

namespace Drupal\earkiv\Drush\Commands;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\Exception\FileException;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\earkiv\ArchiveStatusEnum;
use Drupal\earkiv\EArkivDrushCommandsInterface;
use Drupal\earkiv\Entity\Archive;
use Drupal\earkiv\JobStatusTrait;
use Drupal\earkiv\StoragePluginManager;
use Drush\Commands\DrushCommands;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class LocalStorageCommands.
 */
final class LocalStorageCommands extends DrushCommands implements EArkivDrushCommandsInterface {

  use JobStatusTrait;

  protected EntityTypeManagerInterface $entityTypeManager;
  protected FileSystemInterface $fileSystem;
  protected StoragePluginManager $pluginManager;
  protected LoggerChannelInterface $log;

  /**
   * HTTrackCommands constructor.
   */
  final public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    FileSystemInterface $file_system,
    StoragePluginManager $plugin_manager,
    LoggerChannelFactoryInterface $logger_channel_factory
  ) {
    parent::__construct();
    $this->entityTypeManager = $entity_type_manager;
    $this->fileSystem = $file_system;
    $this->pluginManager = $plugin_manager;
    $this->log = $logger_channel_factory->get('earkiv');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('file_system'),
      $container->get('plugin.manager.earkiv_storage'),
      $container->get('logger.factory')
    );
  }

  /**
   * Process job.
   */
  public function process(Archive $entity, OutputInterface $output): void {
    $this->setOutput($output);

    try {
      // Update Archive entity.
      $entity->set('status', ArchiveStatusEnum::RUNNING_STORE);
      $entity->save();

      /** @var \Drupal\earkiv\Plugin\earkiv\Storage\LocalStorage $storage */
      $storage = $this->pluginManager->createInstance($entity->getStoragePlugin());;
      $storage->setConfiguration($entity->getStorageConfig());

      // Move the archive from temporary storage to the configured directory.
      $file = $entity->getFile();
      $source_path = $file->getFileUri();
      $destination_path = 'private://' . $entity->getStorageConfig()['path'] . '/' . $file->getFilename();
      $this->fileSystem->move($source_path, $destination_path);
      $file->setFileUri($destination_path);
      $file->save();

      // Signal that the job is done.
      $entity->set('status', ArchiveStatusEnum::DONE);
      $entity->save();
    }
    catch (FileException $e) {
      $this->log->error('Failed to move the archive from temporary storage: @message', ['@message' => $e->getMessage()]);
    }
    catch (\Exception $e) {
      $this->log->error('Error storing archive: @message', ['@message' => $e->getMessage()]);
      $this->setJobStatusErrorResume($entity);
    }
  }

}
