<?php

declare(strict_types=1);

namespace Drupal\earkiv\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\earkiv\ArchiveStatusEnum;
use Drupal\file\Entity\File;
use Drupal\file\FileInterface;

/**
 * Defines the Archive entity.
 *
 * @ContentEntityType(
 *   id = "earkiv_archive",
 *   label = @Translation("E-Arkiv Archive"),
 *   handlers = {
 *     "storage_schema" = "Drupal\earkiv\EntityStorageSchema",
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\earkiv\ArchiveListBuilder",
 *     "form" = {
 *       "add" = "Drupal\earkiv\Form\ArchiveForm",
 *       "delete" = "Drupal\earkiv\Form\ArchiveDeleteForm",
 *     }
 *   },
 *   base_table = "earkiv_archive",
 *   admin_permission = "administer earkiv entities",
 *   fieldable = FALSE,
 *   entity_keys = {
 *     "id" = "id",
 *   },
 *   links = {
 *     "delete-form" = "/earkiv/archive/{earkiv_archive}/delete",
 *     "collection" = "/admin/content/earkiv"
 *   }
 * )
 */
class Archive extends ContentEntityBase implements ContentEntityInterface {

  /**
   * Number of times to try to resume a process.
   */
  const NUM_RESUME_ATTEMPTS = 3;

  /**
   * {@inheritdoc}
   */
  public function label(): string {
    /** @var FileSystemInterface $file_system */
    $file_system = \Drupal::service('file_system');
    return $file_system->basename($this->file->value ?? '');
  }

  /**
   * {@inheritdoc}
   */
  public function getStatus(): int {
    return (int) $this->get('status')->value;
  }

  /**
   * Get retries.
   */
  public function getRetries(): int {
    return (int) $this->get('retries')->value;
  }

  /**
   * Get archive plugin.
   */
  public function getArchiverPlugin(): string {
    return $this->get('archiver_plugin')->value;
  }

  /**
   * Get archive plugin configuration.
   */
  public function getArchiverConfig(): array {
    return $this->get('archiver_config')->first()?->getValue() ?? [];
  }

  /**
   * Get storage plugin.
   */
  public function getStoragePlugin(): string {
    return $this->get('storage_plugin')->value;
  }

  /**
   * Get storage plugin configuration.
   */
  public function getStorageConfig(): array {
    return $this->get('storage_config')->first()?->getValue() ?? [];
  }

  /**
   * Get created timestamp.
   */
  public function getCreatedTime(): int {
    return (int) $this->get('created')->value;
  }

  /**
   * Get file.
   */
  public function getFile(): ?FileInterface {
    $id = $this->get('file')->first()->getValue()['target_id'];
    if (is_null($id)) {
      return null;
    }
    return File::load($id);
  }

  /**
   * Get changed timestamp.
   */
  public function getChangedTime(): int {
    return (int) $this->get('changed')->value;
  }

  /**
   * @inheritdoc
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type): array {

    // Standard field, used as unique if primary index.
    $fields['id'] = BaseFieldDefinition::create('integer')
      ->setLabel(new TranslatableMarkup('ID'))
      ->setDescription(new TranslatableMarkup('The ID of the entity.'))
      ->setReadOnly(TRUE);

    // Author, the user who created the archive.
    $fields['uid'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(new TranslatableMarkup('Authored by'))
      ->setDescription(new TranslatableMarkup('The ID of the content author.'))
      ->setDefaultValueCallback('\Drupal\earkiv\Entity\Archive::getCurrentUserId');

    // Status.
    $fields['status'] = BaseFieldDefinition::create('integer')
      ->setLabel(new TranslatableMarkup('Status'))
      ->setDescription(new TranslatableMarkup('The status of the archive.'))
      ->setSettings([
        'unsigned' => TRUE,
        'size' => 'tiny'
      ])
      ->setDefaultValue(ArchiveStatusEnum::NEW);

    // Retries.
    $fields['retries'] = BaseFieldDefinition::create('integer')
      ->setLabel(new TranslatableMarkup('Retries'))
      ->setDescription(new TranslatableMarkup('Number of retries attempts made.'))
      ->setSettings([
        'unsigned' => TRUE,
        'size' => 'tiny'
      ])
      ->setDefaultValue(0);

    // Archiver plugin.
    $fields['archiver_plugin'] = BaseFieldDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Archiver'))
      ->setDescription(new TranslatableMarkup('The archiver plugin used to build the archive.'));

    // Archiver settings.
    $fields['archiver_config'] = BaseFieldDefinition::create('map')
      ->setLabel(new TranslatableMarkup('Archiver configuration'))
      ->setDescription(new TranslatableMarkup('The settings for the archiver plugin.'))
      ->setSettings(['serialize' => TRUE]);

    // Storage plugin.
    $fields['storage_plugin'] = BaseFieldDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Storage'))
      ->setDescription(new TranslatableMarkup('The storage plugin used to store the archive.'));

    // Storage settings.
    $fields['storage_config'] = BaseFieldDefinition::create('map')
      ->setLabel(new TranslatableMarkup('Storage configuration'))
      ->setDescription(new TranslatableMarkup('The settings for the storage plugin.'))
      ->setSettings(['serialize' => TRUE]);

    // File.
    $fields['file'] = BaseFieldDefinition::create('file')
      ->setLabel(new TranslatableMarkup('File'))
      ->setDescription(new TranslatableMarkup('The reference to the generated archive file.'));

    // Created, the timestamp for when the entity was created.
    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(new TranslatableMarkup('Created'))
      ->setDescription(new TranslatableMarkup('The time that the entity was created.'));

    // Changed, the timestamp for when the entity was last updated.
    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(new TranslatableMarkup('Changed'))
      ->setDescription(new TranslatableMarkup('The time that the entity was last updated.'));

    return $fields;
  }

  /**
   * {@inheritDoc}
   */
  public function delete(): void {
    // Delete the file.
    $file = $this->getFile();
    if (!is_null($file)) {
      $file->delete();
    }
    parent::delete();
  }

  /**
   * Get current user ID.
   */
  public static function getCurrentUserId() {
    return \Drupal::currentUser()->id();
  }

}
