<?php

declare(strict_types=1);

namespace Drupal\earkiv\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\earkiv\ArchiveStatusEnum;

/**
 * Defines the Schedule entity.
 *
 * @ContentEntityType(
 *   id = "earkiv_schedule",
 *   label = @Translation("E-Arkiv Schedule"),
 *   handlers = {
 *     "storage_schema" = "Drupal\earkiv\EntityStorageSchema",
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\earkiv\ScheduleListBuilder",
 *     "form" = {
 *       "add" = "Drupal\earkiv\Form\ScheduleForm",
 *       "delete" = "Drupal\earkiv\Form\ScheduleDeleteForm",
 *     }
 *   },
 *   base_table = "earkiv_schedule",
 *   admin_permission = "administer earkiv entities",
 *   fieldable = FALSE,
 *   entity_keys = {
 *     "id" = "id",
 *   },
 *   links = {
 *     "delete-form" = "/earkiv/schedule/{earkiv_schedule}/delete",
 *     "collection" = "/admin/content/earkiv/schedules",
 *   }
 * )
 */
class Schedule extends ContentEntityBase implements ContentEntityInterface {

  /**
   * {@inheritdoc}
   */
  public function label(): string {
    return $this->get('label')->value ?? '';
  }

  /**
   * Get rrule.
   */
  public function getRrule(): string {
    return $this->get('rrule')->value;
  }

  /**
   * Get created timestamp.
   */
  public function getCreatedTime(): int {
    return (int) $this->get('created')->value;
  }

  /**
   * Get changed timestamp.
   */
  public function getChangedTime(): int {
    return (int) $this->get('changed')->value;
  }

  /**
   * Associate an Archive with this Schedule.
   *
   * @param Archive $archive
   * @return void
   */
  public function addArchive(Archive $archive): void {
    try {
      \Drupal::database()->insert('earkiv_schedules_archives')
        ->fields([
          'schedule_id' => $this->id(),
          'archive_id' => $archive->id(),
        ])
        ->execute();
    }
    catch (\Exception $e) {
      \Drupal::logger('earkiv')->error('Failed to add archive to schedule: @message', ['@message' => $e->getMessage()]);
    }
  }

  /**
   * Get all Archives associated with this Schedule.
   *
   * @return Archive[]
   */
  public function getArchives(): array {
    $query = \Drupal::database()->select('earkiv_schedules_archives', 'sa')
      ->condition('sa.schedule_id', $this->id());
    $query->addField('sa', 'archive_id');
    $ids = $query->execute()->fetchCol();
    return Archive::loadMultiple($ids);
  }

  /**
   * @inheritdoc
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type): array {

    // Standard field, used as unique if primary index.
    $fields['id'] = BaseFieldDefinition::create('integer')
      ->setLabel(new TranslatableMarkup('ID'))
      ->setDescription(new TranslatableMarkup('The ID of the entity.'))
      ->setReadOnly(TRUE);

    // Author, the user who created the archive.
    $fields['uid'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(new TranslatableMarkup('Authored by'))
      ->setDescription(new TranslatableMarkup('The ID of the content author.'))
      ->setDefaultValueCallback('\Drupal\earkiv\Entity\Schedule::getCurrentUserId');

    // Label.
    $fields['label'] = BaseFieldDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Label'))
      ->setDescription(new TranslatableMarkup('The label of the schedule.'))
      ->setRequired(TRUE);

    // The rrule of the schedule.
    $fields['rrule'] = BaseFieldDefinition::create('string')
      ->setLabel(new TranslatableMarkup('RRule'))
      ->setDescription(new TranslatableMarkup('The RRule of the schedule.'))
      ->setRequired(TRUE);

    // Status.
    $fields['status'] = BaseFieldDefinition::create('integer')
      ->setLabel(new TranslatableMarkup('Status'))
      ->setDescription(new TranslatableMarkup('The status of the schedule.'))
      ->setSettings([
        'unsigned' => TRUE,
        'size' => 'tiny'
      ])
      ->setDefaultValue(1);

    // Created, the timestamp for when the entity was created.
    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(new TranslatableMarkup('Created'))
      ->setDescription(new TranslatableMarkup('The time that the entity was created.'));

    // Changed, the timestamp for when the entity was last updated.
    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(new TranslatableMarkup('Changed'))
      ->setDescription(new TranslatableMarkup('The time that the entity was last updated.'));

    return $fields;
  }

  /**
   * Get current user ID.
   */
  public static function getCurrentUserId() {
    return \Drupal::currentUser()->id();
  }

}
