<?php

namespace Drupal\earkiv\Form;

use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformState;
use Drupal\earkiv\PluginOptionsTrait;

/**
 * Form controller for the 'earkiv_archive' entity edit forms.
 */
class ArchiveForm extends ContentEntityForm {

  use PluginOptionsTrait;

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);

    $input = $form_state->getUserInput();
    $config = $this->config('earkiv.settings');

    $archiver = $form_state->getValue('archiver_plugin') ?? $config->get('default_archiver_plugin');
    $form['archiver_plugin'] = [
      '#type' => 'select',
      '#title' => $this->t('Archiver'),
      '#description' => $this->t('Select the archiver plugins to use.'),
      '#options' => $this->getArchiverPluginOptions(),
      '#default_value' => $archiver,
      '#required' => TRUE,
      '#ajax' => [
        'callback' => '::archiverConfigCallback',
        'disable-refocus' => FALSE,
        'event' => 'change',
        'wrapper' => 'archiver-config-wrapper',
        'progress' => [
          'type' => 'throbber',
          'message' => $this->t('Loading...'),
        ],
        '#limit_validation_errors' => [],
      ],
    ];

    // Get the config form for the selected archiver.
    if ($archiver) {
      $plugin = $this->getArchiverPluginManager()->createInstance($archiver);
      $plugin->setConfiguration($config->get($archiver) ?? []);
      $form['archiver_config'] = [
        '#type' => 'details',
        '#title' => $this->t('@label archiver configuration', ['@label' => $plugin?->getLabel()]),
        '#tree' => TRUE,
        '#prefix' => '<div id="archiver-config-wrapper">',
        '#suffix' => '</div>',
      ];
      $form['archiver_config'] += $plugin->buildConfigurationForm([], $form_state);
    }
    else {
      $form['archiver_config'] = [
        '#type' => 'details',
        '#title' => $this->t('Archiver configuration'),
        '#title_display' => 'invisible',
        '#prefix' => '<div id="archiver-config-wrapper">',
        '#suffix' => '</div>',
      ];
      $form['archiver_config']['message'] = [
        '#markup' => $this->t('No archiver plugin selected.'),
      ];
    }

    $storage = $form_state->getValue('storage_plugin') ?? $config->get('default_storage_plugin');
    $form['storage_plugin'] = [
      '#type' => 'select',
      '#title' => $this->t('Storage'),
      '#description' => $this->t('Select the storage plugin to use.'),
      '#options' => $this->getStoragePluginOptions(),
      '#default_value' => $storage,
      '#required' => TRUE,
      '#ajax' => [
        'callback' => '::storageConfigCallback',
        'disable-refocus' => FALSE,
        'event' => 'change',
        'wrapper' => 'storage-config-wrapper',
        'progress' => [
          'type' => 'throbber',
          'message' => $this->t('Loading...'),
        ],
        '#limit_validation_errors' => [],
      ],
    ];

    // Get the config form for the selected storage.
    if ($storage) {
      $plugin = $this->getStoragePluginManager()->createInstance($storage);
      $plugin->setConfiguration($config->get($storage) ?? []);
      $form['storage_config'] = [
        '#type' => 'details',
        '#title' => $this->t('@label storage configuration', ['@label' => $plugin?->getLabel()]),
        '#tree' => TRUE,
        '#prefix' => '<div id="storage-config-wrapper">',
        '#suffix' => '</div>',
      ];
      $form['storage_config'] += $plugin->buildConfigurationForm([], $form_state);
    }
    else {
      $form['storage_config'] = [
        '#type' => 'details',
        '#title' => $this->t('Storage configuration'),
        '#title_display' => 'invisible',
        '#prefix' => '<div id="storage-config-wrapper">',
        '#suffix' => '</div>',
      ];
      $form['storage_config']['message'] = [
        '#markup' => $this->t('No storage plugin selected.'),
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    parent::validateForm($form, $form_state);
    /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
    $entity = $this->buildEntity($form, $form_state);

    // Validate archiver plugin.
    if ($form_state->getValue('archiver_plugin', FALSE)) {
      $plugin_manager = \Drupal::service('plugin.manager.earkiv_archiver');
      $plugin_manager->createInstance($form_state->getValue('archiver_plugin'))
        ->validateConfigurationForm($form['archiver_config'], SubformState::createForSubform($form['archiver_config'], $form, $form_state));
    }

    // Validate storage plugin.
    if ($form_state->getValue('storage_plugin', FALSE)) {
      $plugin_manager = \Drupal::service('plugin.manager.earkiv_storage');
      $plugin_manager->createInstance($form_state->getValue('storage_plugin'))
        ->validateConfigurationForm($form['storage_config'], SubformState::createForSubform($form['storage_config'], $form, $form_state));
    }
  }

  /**
   * Ajax callback for archiver config.
   */
  public function archiverConfigCallback(array &$form, FormStateInterface $form_state): array {
    return $form['archiver_config'];
  }

  /**
   * Ajax callback for storage config.
   */
  public function storageConfigCallback(array &$form, FormStateInterface $form_state): array {
    return $form['storage_config'];
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state): int {
    $status = parent::save($form, $form_state);

    $entity = $this->entity;
    if ($status == SAVED_UPDATED) {
      $this->messenger()
        ->addMessage($this->t('The Archive %archive has been updated.', ['%archive' => $entity->label()]));
    } else {
      $this->messenger()
        ->addMessage($this->t('The Archive %archive has been created.', ['%archive' => $entity->label()]));
    }

    $form_state->setRedirectUrl($this->entity->toUrl('collection'));
    return $status;
  }
}
