<?php

declare(strict_types=1);

namespace Drupal\earkiv\Form;

use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformState;
use Drupal\earkiv\PluginOptionsTrait;

/**
 * Provides a settings form for the E-Arkiv configuration.
 */
class SettingsForm extends ConfigFormBase {

  use PluginOptionsTrait;

  /**
   * {@inheritDoc}
   */
  public function getFormId(): string {
    return 'earkiv_settings';
  }

  /**
   * {@inheritDoc}
   */
  public function getEditableConfigNames(): array {
    return ['earkiv.settings'];
  }

  /**
   * {@inheritDoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildForm($form, $form_state);
    $config = $this->config('earkiv.settings');

    $archiver = $config->get('default_archiver_plugin');
    $form['default_archiver_plugin'] = [
      '#type' => 'select',
      '#title' => $this->t('Default archiver'),
      '#description' => $this->t('Select the default archiver plugins to use.'),
      '#options' => $this->getArchiverPluginOptions(),
      '#default_value' => $archiver,
      '#required' => TRUE,
    ];

    $storage = $config->get('default_storage_plugin');
    $form['default_storage_plugin'] = [
      '#type' => 'select',
      '#title' => $this->t('Default storage'),
      '#description' => $this->t('Select the default storage plugin to use.'),
      '#options' => $this->getStoragePluginOptions(),
      '#default_value' => $storage,
      '#required' => TRUE,
    ];

    // Get the config form for the selected archiver.
    if ($archiver) {
      $plugin = $this->getArchiverPluginManager()->createInstance($archiver);
      $plugin->setConfiguration($config->get($archiver) ?? []);
      $form[$archiver] = [
        '#type' => 'details',
        '#title' => $this->t('@label archiver default configuration', ['@label' => $plugin->getLabel()]),
        '#tree' => TRUE,
      ];
      $form[$archiver] += $plugin->buildConfigurationForm([], $form_state);
    }

    // Get the config form for the selected storage.
    if ($storage) {
      $plugin = $this->getStoragePluginManager()->createInstance($storage);
      $plugin->setConfiguration($config->get($storage) ?? []);
      $form[$storage] = [
        '#type' => 'details',
        '#title' => $this->t('@label storage default configuration', ['@label' => $plugin->getLabel()]),
        '#tree' => TRUE,
      ];
      $form[$storage] += $plugin->buildConfigurationForm([], $form_state);
    }

    return $form;
  }

  /**
   * {@inheritDoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    parent::validateForm($form, $form_state);

    // Validate archiver plugin.
    $plugin_manager = \Drupal::service('plugin.manager.earkiv_archiver');
    $plugin = $form_state->getValue('default_archiver_plugin');
    $this->validatePlugin($plugin_manager, $plugin, $form, $form_state);

    // Validate storage plugin.
    $plugin_manager = \Drupal::service('plugin.manager.earkiv_storage');
    $plugin = $form_state->getValue('default_storage_plugin');
    $this->validatePlugin($plugin_manager, $plugin, $form, $form_state);
  }

  /**
   * Validate plugin subform.
   */
  protected function validatePlugin(PluginManagerInterface $plugin_manager, string $plugin_id, array &$form, FormStateInterface $form_state) {
    if (empty($form[$plugin_id])) return;
    $subform = $form[$plugin_id];
    $plugin_manager->createInstance($plugin_id)->validateConfigurationForm($subform, SubformState::createForSubform($subform, $form, $form_state));
  }

  /**
   * {@inheritDoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $values = $form_state->getValues();
    $config = $this->configFactory()->getEditable('earkiv.settings');
    $config->set('default_archiver_plugin', $values['default_archiver_plugin'])
      ->set('default_storage_plugin', $values['default_storage_plugin']);
    if ($values['default_archiver_plugin']) {
      $plugin_config = $form_state->getValue($values['default_archiver_plugin']);
      if (is_array($plugin_config) ) {
        foreach ($plugin_config as $key => $value) {
          $config->set($values['default_archiver_plugin'] . '.' . $key, $value);
        }
      }
    }
    if ($values['default_storage_plugin']) {
      $plugin_config = $form_state->getValue($values['default_storage_plugin']);
      if (is_array($plugin_config) ) {
        foreach ($plugin_config as $key => $value) {
          $config->set($values['default_storage_plugin'] . '.' . $key, $value);
        }
      }
    }
    $config->save();
  }

}
