<?php

declare(strict_types=1);

namespace Drupal\earkiv\Hook;

use Drupal\Component\Utility\Html;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\earkiv\ArchiverPluginManager;
use Drupal\earkiv\StoragePluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Represents a central place to manage all uncategorized hooks.
 */
class Hooks implements ContainerInjectionInterface {

  protected ModuleHandlerInterface $moduleHandler;
  protected ConfigFactoryInterface $configFactory;
  protected ArchiverPluginManager $archiverPluginManager;
  protected StoragePluginManager $storagePluginManager;

  /**
   * Hooks constructor.
   */
  public function __construct(
    ModuleHandlerInterface $module_handler,
    ConfigFactoryInterface $config_factory,
    ArchiverPluginManager $archiver_plugin_manager,
    StoragePluginManager $storage_plugin_manager
  ) {
    $this->moduleHandler = $module_handler;
    $this->configFactory = $config_factory;
    $this->archiverPluginManager = $archiver_plugin_manager;
    $this->storagePluginManager = $storage_plugin_manager;
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('module_handler'),
      $container->get('language_manager'),
      $container->get('plugin.manager.earkiv_archiver'),
      $container->get('plugin.manager.earkiv_storage')
    );
  }

  /**
   * Implements hook_schema().
   *
   * This hook is not supported by the new hook annotation, so it is invoked via the legacy hook system.
   */
  public function schema(): array {
    $schema['earkiv_schedules_archives'] = [
      'description' => 'Stores relations between {earkiv_schedule} and {earkiv_archive} entities.',
      'fields' => [
        'schedule_id' => [
          'description' => 'The {earkiv_schedule}.id this record affects.',
          'type' => 'int',
          'unsigned' => TRUE,
          'not null' => TRUE,
          'default' => 0,
        ],
        'archive_id' => [
          'description' => 'The {earkiv_archive}.id this record affects.',
          'type' => 'int',
          'unsigned' => TRUE,
          'not null' => TRUE,
          'default' => 0,
        ],
      ],
      'primary key' => [
        'schedule_id',
        'archive_id',
      ],
    ];
    return $schema;
  }

  /**
   * Implements hook_requirements().
   *
   * This hook is not supported by the new hook annotation, so it is invoked via the legacy hook system.
   */
  public function requirements(string $phase): array {
    $requirements = [];
    $config = $this->configFactory->get('earkiv.settings');

    // Get archiver plugin requirements.
    $archiver = $config->get('archiver');
    if ($archiver) {
      $plugin = $this->archiverPluginManager->createInstance($archiver);
      $requirements += $plugin->requirements($phase);
    }

    // Get storage plugin requirements.
    $storage = $config->get('storage');
    if ($storage) {
      $plugin = $this->storagePluginManager->createInstance($storage);
      $requirements += $plugin->requirements($phase);
    }

    return $requirements;
  }

  /**
   * Implements hook_help().
   */
  #[Hook('help')]
  public function help($route_name, RouteMatchInterface $route_match): string {
    if ($route_name == 'help.page.earkiv') {
      $text = file_get_contents(__DIR__ . '../../../README.md');
      if (!$this->moduleHandler->moduleExists('markdown')) {
        return '<pre>' . Html::escape($text) . '</pre>';
      }
      else {
        // Use the Markdown filter to render the README.
        $filter_manager = \Drupal::service('plugin.manager.filter');
        $settings = $this->configFactory->get('markdown.settings')->getRawData();
        $config = ['settings' => $settings];
        $filter = $filter_manager->createInstance('markdown', $config);
        return $filter->process($text, 'en');
      }
    }
    return '';
  }

}
