<?php

declare(strict_types=1);

namespace Drupal\earkiv\Plugin\earkiv\Archiver;

use Drupal\Core\Form\FormStateInterface;
use Drupal\earkiv\ArchiverBase;
use Drupal\earkiv\Drush\Commands\HTTrackCommands;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Provides E-Arkiv HTTrack archiver plugin.
 *
 * @Archiver(
 *   id = "httrack_archiver",
 *   label = @Translation("HTTrack"),
 *   description = @Translation("E-Arkiv HTTrack archiver plugin.")
 * )
 */
class HTTrackArchiver extends ArchiverBase {

  protected RequestStack $requestStack;

  /**
   * HTTrackArchiver constructor.
   */
  public function __construct(
    array $configuration,
          $plugin_id,
          $plugin_definition,
    RequestStack $request_stack
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->requestStack = $request_stack;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('request_stack')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form['url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('URL'),
      '#description' => $this->t('The full URL of the site to archive.'),
      '#default_value' => $this->configuration['url'] ?? $this->requestStack->getCurrentRequest()->getSchemeAndHttpHost(),
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $valid = filter_var($form_state->getValue('url'), FILTER_VALIDATE_URL);
    if ($valid === FALSE) {
      $form_state->setErrorByName('url', $this->t('The URL is not valid.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $this->configuration['url'] = $form_state->getValue('url');
  }

  /**
   * {@inheritdoc}
   */
  public function requirements(string $phase): array {
    $requirements = [];

    if ($phase == 'runtime') {
      // Check for httrack command.
      $return = shell_exec('command httrack --version') ?? '';
      if (empty($return) || str_contains($return, 'command not found')) {
        $requirements['httrack'] = [
          'title' => 'HTTrack',
          'value' => $this->t('HTTrack is required by the <em>E-Arkiv</em> archiver plugin <em>HTTrack</em>. Please see <a href="https://www.httrack.com/">official website</a> for installation instructions.'),
          'severity' => REQUIREMENT_ERROR,
        ];
      }
      else {
        $requirements['httrack'] = [
          'title' => 'HTTrack',
          'value' => $return,
          'severity' => REQUIREMENT_INFO,
        ];
      }

      // Check for tar command.
      $return = shell_exec('command tar --version');
      if (str_contains($return, 'command not found')) {
        $requirements['tar'] = [
          'title' => 'tar',
          'value' => $this->t('<code>tar</code> command line tool is required by the <em>E-Arkiv</em> archiver plugin <em>HTTrack</em>.'),
          'severity' => REQUIREMENT_ERROR,
        ];
      }

      // Check for gzip command.
      $return = shell_exec('command gzip --version');
      if (str_contains($return, 'command not found')) {
        $requirements['gzip'] = [
          'title' => 'gzip',
          'value' => $this->t('<code>gzip</code> command line tool is required by the <em>E-Arkiv</em> archiver plugin <em>HTTrack</em>.'),
          'severity' => REQUIREMENT_ERROR,
        ];
      }
    }

    return $requirements;
  }

  /**
   * {@inheritDoc}
   */
  public function getDrushCommands(): HTTrackCommands {
    return HTTrackCommands::create(\Drupal::getContainer());
  }

}
