<?php

declare(strict_types=1);

namespace Drupal\earkiv\Plugin\earkiv\Storage;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\Exception\FileException;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\earkiv\Drush\Commands\LocalStorageCommands;
use Drupal\earkiv\Entity\Archive;
use Drupal\earkiv\StorageBase;
use Drupal\file\Entity\File;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides E-Arkiv local storage plugin.
 *
 * @Storage(
 *   id = "local_storage",
 *   label = @Translation("Local"),
 *   description = @Translation("E-Arkiv local storage plugin.")
 * )
 */
class LocalStorage extends StorageBase {

  protected EntityTypeManagerInterface $entityTypeManager;
  protected ConfigFactoryInterface $configFactory;
  protected FileSystemInterface $fileSystem;

  /**
   * LocalStorage constructor.
   */
  public function __construct(
    array $configuration,
          $plugin_id,
          $plugin_definition,
    EntityTypeManagerInterface $entity_type_manager,
    ConfigFactoryInterface $config_factory,
    FileSystemInterface $file_system
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->configFactory = $config_factory;
    $this->fileSystem = $file_system;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('config.factory'),
      $container->get('file_system')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    // TODO: Rename this to 'directory/dir' perhaps?
    $form['path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Directory'),
      '#description' => $this->t('The name of the directory in the private file system where the archives will be stored.'),
      '#default_value' => $this->configuration['path'] ?? 'earkiv',
      '#required' => TRUE,
    ];

    if (!\Drupal::hasService('stream_wrapper.private')) {
      $form['error'] = [
        '#prefix' => '<span class="color-error">',
        '#suffix' => '</span>',
        '#markup' => $this->t('The private file system is not configured.'),
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state): void {
    if (!\Drupal::hasService('stream_wrapper.private')) {
      $form_state->setErrorByName('path', $this->t('The private file system is not configured.'));
    }

    $directory = 'private://' . trim(($form_state->getValue('path') ?? ''), '/');
    $real_path = $this->fileSystem->realpath($directory);
    $status = $this->fileSystem->prepareDirectory($real_path, FileSystemInterface::CREATE_DIRECTORY);
    if ($status === FALSE) {
      $form_state->setErrorByName('path', $this->t('The specified directory does not exist or is not writable.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $this->configuration['path'] = trim($form_state->getValue('path'), '/');
  }

  /**
   * {@inheritdoc}
   */
  public function requirements(string $phase): array {
    $requirements = [];

    if ($phase == 'runtime') {
      // Check if the private file system is configured.
      if (!\Drupal::hasService('stream_wrapper.private')) {
        $requirements['private_file_system'] = [
          'title' => 'Private file system',
          'value' => $this->t('You need to configure the private file system to use the <em>E-Arkiv</em> storage plugin <em>Local storage</em>.'),
          'severity' => REQUIREMENT_ERROR,
        ];
      }
    }

    return $requirements;
  }

  /**
   * {@inheritDoc}
   */
  public function getDrushCommands(): LocalStorageCommands {
    return LocalStorageCommands::create(\Drupal::getContainer());
  }

}
