<?php

declare(strict_types=1);

namespace Drupal\earkiv;

use Drupal\Component\Plugin\Exception\PluginException;

/**
 * Trait PluginOptionsTrait.
 *
 * Provides methods to manage and retrieve plugin options for archiver and
 * storage plugins.
 */
trait PluginOptionsTrait {

  protected ?ArchiverPluginManager $archiverPluginManager = NULL;
  protected ?StoragePluginManager $storagePluginManager = NULL;

  public function getArchiverPluginManager(): ArchiverPluginManager {
    if (!$this->archiverPluginManager) {
      $this->archiverPluginManager = \Drupal::service('plugin.manager.earkiv_archiver');
    }
    return $this->archiverPluginManager;
  }

  public function getStoragePluginManager(): StoragePluginManager {
    if (!$this->storagePluginManager) {
      $this->storagePluginManager = \Drupal::service('plugin.manager.earkiv_storage');
    }
    return $this->storagePluginManager;
  }

  /**
   * Builds the archiver plugin options.
   *
   * @param bool $include_empty
   *    (optional) Whether to include an empty option. Defaults to TRUE.
   * @return array
   *   An array of available plugins.
   * @throws PluginException
   */
  protected function getArchiverPluginOptions(bool $include_empty = TRUE): array {
    $options = [];
    if ($include_empty) {
      $options[''] = $this->t('Select an archiver');
    }
    $all_plugins = $this->getArchiverPluginManager()->getDefinitions();
    uasort($all_plugins, function ($a, $b) {
      return strnatcasecmp((string)$a['label'], (string)$b['label']);
    });
    foreach ($all_plugins as $definition) {
      /** @var \Drupal\earkiv\ArchiverInterface $plugin */
      $plugin = $this->getArchiverPluginManager()->createInstance($definition['id']);
      $options[$plugin->getPluginId()] = $plugin->getLabel();
    }
    return $options;
  }

  /**
   * Builds the storage plugin options.
   *
   * @param bool $include_empty
   *   (optional) Whether to include an empty option. Defaults to TRUE.
   * @return array
   *   An array of available plugins.
   * @throws PluginException
   */
  protected function getStoragePluginOptions(bool $include_empty = TRUE): array {
    $options = [];
    if ($include_empty) {
      $options[''] = $this->t('Select an archiver');
    }
    $all_plugins = $this->getStoragePluginManager()->getDefinitions();
    uasort($all_plugins, function ($a, $b) {
      return strnatcasecmp((string)$a['label'], (string)$b['label']);
    });
    foreach ($all_plugins as $definition) {
      /** @var \Drupal\earkiv\StorageInterface $plugin */
      $plugin = $this->getStoragePluginManager()->createInstance($definition['id']);
      $options[$plugin->getPluginId()] = $plugin->getLabel();
    }
    return $options;
  }
}
